
/* cmd1.c - functions implementing score processing statements */

#include "main.h"
#include "cmd.h"

/* set printing format */

void	cmd_pformat (int nr_args, double *arg_list)
{
	int	p;

	if (nr_args < 2) err_msg_s ("missing argument for pformat", NULL);
	if (nr_args > 5) err_msg_s ("too many arguments for pformat", NULL);
	p = check_int (arg_list[0], 1, P_MAX, "invalid p-field number") - 1;
	p_params[p].fwidth =
		check_int (arg_list[1], 1, 40, "invalid field width");
	if (nr_args < 3) return;
	p_params[p].precision =
		check_int (arg_list[2], 0, 14, "invalid precision");
	if (nr_args < 4) return;
	p_params[p].l_limit = arg_list[3];
	if (nr_args < 5) return;
	p_params[p].u_limit = arg_list[4];
}

/* set event mask */

void	cmd_mask (int nr_args, double *arg_list)
{
	int	m;

	if (nr_args < 2) err_msg_s ("missing argument for mask", NULL);
	if (nr_args > 6) err_msg_s ("too many arguments for mask", NULL);
	m = check_int (arg_list[0], 0, (N_MASKS - 1), "invalid mask number");
	e_masks[m].m_mode = check_int (arg_list[1], 0, 2, "invalid mask mode");
	if ((nr_args < 3) && (e_masks[m].m_mode == 0)) return;
	if (nr_args != 6) err_msg_s ("missing argument for mask", NULL);
	e_masks[m].p_num = check_int (arg_list[2], 0, P_MAX,
					"invalid p-field number");
	e_masks[m].l_limit = arg_list[3];
	e_masks[m].u_limit = arg_list[4];
	e_masks[m].loop = arg_list[5];
}

/* set a p-field to the specified value */

void	cmd_set (int nr_args, double *arg_list)
{
	int	e, p;
	double	x;

	if (nr_args < 2) err_msg_s ("missing argument for set", NULL);
	if (nr_args > 2) err_msg_s ("too many arguments for set", NULL);
	p = check_int (arg_list[0], 0, P_MAX, "invalid p-field number");
	x = arg_list[1];
	e = -1; while ((e = next_event (e)) >= 0) {
		store_pfield (x, e, p);
	}
}

/* add to p-field */

void	cmd_add (int nr_args, double *arg_list)
{
	int	e, p;
	double	x, y;

	if (nr_args < 2) err_msg_s ("missing argument for add", NULL);
	if (nr_args > 2) err_msg_s ("too many arguments for add", NULL);
	p = check_int (arg_list[0], 0, P_MAX, "invalid p-field number");
	y = arg_list[1];
	e = -1; while ((e = next_event (e)) >= 0) {
		if (read_pfield (&x, e, p)) continue;
		x += y;
		store_pfield (x, e, p);
	}
}

/* multiply */

void	cmd_mul (int nr_args, double *arg_list)
{
	int	e, p;
	double	x, y;

	if (nr_args < 2) err_msg_s ("missing argument for mul", NULL);
	if (nr_args > 2) err_msg_s ("too many arguments for mul", NULL);
	p = check_int (arg_list[0], 0, P_MAX, "invalid p-field number");
	y = arg_list[1];
	e = -1; while ((e = next_event (e)) >= 0) {
		if (read_pfield (&x, e, p)) continue;
		x *= y;
		store_pfield (x, e, p);
	}
}

/* raise p-field to the specified power */

void	cmd_pow (int nr_args, double *arg_list)
{
	int	e, p;
	double	x, y;

	if (nr_args < 2) err_msg_s ("missing argument for pow", NULL);
	if (nr_args > 2) err_msg_s ("too many arguments for pow", NULL);
	p = check_int (arg_list[0], 0, P_MAX, "invalid p-field number");
	y = arg_list[1];
	e = -1; while ((e = next_event (e)) >= 0) {
		if (read_pfield (&x, e, p)) continue;
		x = pow (x, y);
		store_pfield (x, e, p);
	}
}

/* quantize */

void	cmd_quant (int nr_args, double *arg_list)
{
	int	e, p;
	double	x, q, s, a;

	if (nr_args < 2) err_msg_s ("missing argument for quant", NULL);
	if (nr_args > 3) err_msg_s ("too many arguments for quant", NULL);
	p = check_int (arg_list[0], 0, P_MAX, "invalid p-field number");
	s = arg_list[1];
	a = 0.0; if (nr_args > 2) a = arg_list[2];
	check_value (s, 1.0e-12, 1.0e+12, "invalid snap value");
	check_value (a, -1.0e+6, 1.0e+6, "invalid quantize amount");
	e = -1; while ((e = next_event (e)) >= 0) {
		if (read_pfield (&x, e, p)) continue;
		q = floor ((x / s) + 0.5) * s;
		x = q + (x - q) * a;
		store_pfield (x, e, p);
	}
}

/* limit */

void	cmd_limit (int nr_args, double *arg_list)
{
	int	e, p;
	double	x, l1, l2;

	if (nr_args < 3) err_msg_s ("missing argument for limit", NULL);
	if (nr_args > 3) err_msg_s ("too many arguments for limit", NULL);
	p = check_int (arg_list[0], 0, P_MAX, "invalid p-field number");
	l1 = arg_list[1]; l2 = arg_list[2];
	e = -1; while ((e = next_event (e)) >= 0) {
		if (read_pfield (&x, e, p)) continue;
		if (l1 > l2) {
			x = (l1 + l2) * 0.5;
		} else if (x < l1) {
			x = l1;
		} else if (x > l2) {
			x = l2;
		}
		store_pfield (x, e, p);
	}
}

