
/* cmd2.c - functions implementing score processing statements */

#include "main.h"
#include "cmd.h"

unsigned long	r_seed;			/* random seed			*/

/* Return a random number between -1.0 and 1.0 */

double	get_rand (void)
{
	unsigned long	xl, xh;
	double		x;

	/* x = (16807 * x) % 0x7FFFFFFF */

	xl = (r_seed & 0xFFFFUL) * 16807UL; xh = (r_seed >> 16) * 16807UL;
	r_seed = xl + ((xh & 0x7FFFUL) << 16) + (xh >> 15);
	if (r_seed >= 0x7FFFFFFFUL) r_seed = (r_seed + 1UL) & 0x7FFFFFFFUL;

	/* convert to double */

	x = ((double) r_seed - (double) 0x40000000L) / (double) 0x3FFFFFFFL;

	return x;
}

/* set random seed */

void	cmd_seed (int nr_args, double *arg_list)
{
	int		s, l;
	unsigned long	l2;
#ifdef USE_GETTIMEOFDAY
	struct timeval	t;
#endif

	if (nr_args < 1) err_msg_s ("missing argument for seed", NULL);
	if (nr_args > 1) err_msg_s ("too many arguments for seed", NULL);
	l = INT_MAX; l2 = 0x7FFFFFFEUL;
	if ((unsigned long) l > l2) l = (int) l2;
	s = check_int (arg_list[0], 0, l, "invalid seed value");
	if (s) {
		r_seed = (unsigned long) s;
	} else {				/* seed from current time */
#ifdef USE_GETTIMEOFDAY
		gettimeofday (&t, NULL);
		r_seed = ((unsigned long) t.tv_sec % 4294UL) * 1000000UL;
		if (r_seed > 0UL) r_seed -= 32704UL;
		r_seed += (unsigned long) t.tv_usec;
#else
		r_seed = (unsigned long) time (NULL);
#endif
		r_seed = (r_seed % l2) + 1UL;
		get_rand ();
	}
}

