
/* scoreproc - score processing utility	*/
/* written by Istvan Varga, Jul 2001	*/

#include "main.h"

char		**argv;			/* command line arguments	*/
FILE		*in_file;		/* score file			*/
FILE		*cmd_file;		/* program file			*/
FILE		*out_file;		/* output file			*/
char		*in_file_name;		/* score file name		*/
char		*cmd_file_name;		/* program file name		*/
char		*out_file_name;		/* output file name		*/
int		nr_events;		/* number of events		*/
score_event	*event_list;		/* list of score events		*/
p_format	p_params[P_MAX];	/* score printing parameters	*/

int	main (int argc, char **argv_main)
{
	int	j;

	/* default settings */

	argv = argv_main;
	in_file = stdin; in_file_name = NULL;
	cmd_file = NULL; cmd_file_name = NULL;
	out_file = stdout; out_file_name = NULL;
	nr_events = 0; event_list = NULL;
	cmd_line_num = 0;
	init_p_params ();

	/* command line arguments */

	j = 0; while (++j < argc) {
		if ((!(strcmp (argv[j], "-h")))		/* print help */
		    || (!(strcmp (argv[j], "-help")))
		    || (!(strcmp (argv[j], "--help")))) {
		    	print_usage ();
			exit (0);
		} else if ((!(strcmp (argv[j], "-i")))	/* input file */
			   || (!(strcmp (argv[j], "-infile")))) {
			if (++j >= argc)
				err_msg ("missing command line argument", -1);
			in_file_name = argv[j];
		} else if ((!(strcmp (argv[j], "-c")))	/* program file */
			   || (!(strcmp (argv[j], "-cmdfile")))) {
			if (++j >= argc)
				err_msg ("missing command line argument", -1);
			cmd_file_name = argv[j];
		} else if ((!(strcmp (argv[j], "-o")))	/* output file */
			   || (!(strcmp (argv[j], "-outfile")))) {
			if (++j >= argc)
				err_msg ("missing command line argument", -1);
			out_file_name = argv[j];
		} else {
			err_msg ("invalid command line option", j);
		}
	}

	/* open files */

	if (in_file_name != NULL) {
		in_file = fopen (in_file_name, "rt");
		if (in_file == NULL)
			err_msg_s ("error opening file", in_file_name);
	}
	if (cmd_file_name != NULL) {
		cmd_file = fopen (cmd_file_name, "rt");
		if (cmd_file == NULL)
			err_msg_s ("error opening file", cmd_file_name);
	}
	if (out_file_name != NULL) {
		out_file = fopen (out_file_name, "wt");
		if (out_file == NULL)
			err_msg_s ("error opening file", out_file_name);
	}

	/* read score events */

	read_score ();

	/* read program file */

	if (cmd_file != NULL) cmd_main ();

	/* print new event list */

	print_score ();

	/* free event list */

	free_list ();

	/* close files */

	if ((in_file_name != NULL) && (fclose (in_file)))
		err_msg_s ("error closing file", in_file_name);
	if ((cmd_file_name != NULL) && (fclose (cmd_file)))
		err_msg_s ("error closing file", cmd_file_name);
	if (out_file_name != NULL) {
		fflush (out_file);
		if (fclose (out_file))
			err_msg_s ("error closing file", out_file_name);
	}

	return 0;
}

