
/* read_file.c - functions to parse text files */

#include "main.h"

/* Read a line from file "f", store arguments separated by whitespace	*/
/* in "args". Return value is the number of arguments read (or -1 at	*/
/* end of file).							*/

int	get_line (FILE *f, char ***args)
{
	int	end_l, end_f, c, n_chr, n_args, size_chr, size_args;

	if (feof (f) || ferror (f)) return -1;	/* end of file */

	end_l = end_f = n_args = size_args = 0;
	*args = NULL;

	do {				/* allocate space for arguments */
		if (n_args > (size_args - 2)) {
			size_args += 64;
			*args = (char**) realloc ((void*) *args,
						  sizeof (char*) * size_args);
			if (*args == NULL) err_msg ("not enough memory", -1);
		}
		n_chr = size_chr = 0;
		(*args)[n_args] = NULL;
		do {
			c = getc (f);			/* skip whitespace */
		} while ((c == ' ') || (c == '\t'));
		do {
			if (c == '\n') end_l = 1;	/* end of line */
			if (c == EOF) end_f = 1;	/* end of file */
			if ((c == '#') || (c == ';')) {	/* comment */
				end_l = 1;
				do {			/* skip rest of line */
					c = getc (f);
				} while ((c != '\n') && (c != EOF));
				if (c == EOF) end_f = 1;
			}
			if ((end_l) || (end_f)) {	/* end of line/file */
				if ((!(n_chr)) && ((*args)[n_args] != NULL)) {
					free ((*args)[n_args]);	/* nothing */
					(*args)[n_args] = NULL;	/* was read */
				}
				break;
			}
			if (n_chr > (size_chr - 2)) {	/* extend buffer */
				size_chr += 64;
				(*args)[n_args] = (char*) realloc
						  ((void*) (*args)[n_args],
						   sizeof (char) * size_chr);
				if ((*args)[n_args] == NULL)
					err_msg ("not enough memory", -1);
			}
			((*args)[n_args])[n_chr++] = (char) c;
			c = getc (f);
		} while ((c != ' ') && (c != '\t'));
		if (n_chr) {	/* terminate string and read next argument */
			((*args)[n_args])[n_chr] = (char) 0;
			n_args++;
		}
	} while (!((end_l) || (end_f)));
	if (!(n_args)) {
		if (*args != NULL) {			/* free memory if   */
			free (*args); *args = NULL;	/* nothing was read */
		}
		if (end_f) return -1;			/* end of file */
	} else {
		(*args)[n_args] = NULL;	/* terminate string list with NULL */
	}

	return n_args;		/* return the number of arguments read */
}

/* Read a statement with parameters from text file. The statement is stored */
/* in stat (NULL if nothing was read), and the parameters are stored in     */
/* args. Return value is the number of parameters, or -1 for end of file.   */

int	get_statement (FILE *f, char **stat, double **args)
{
	int	i, pcnt;
	char	**arg_list;

	*stat = NULL; *args = NULL;

	/* read a line from input file */

	pcnt = get_line (f, &arg_list);
	if (pcnt < 1) return pcnt;	/* end of file or nothing was read */

	*stat = arg_list[0];
	if (--pcnt) {
		*args = (double*) malloc (sizeof (double) * pcnt);
		for (i = 0; i < pcnt; i++) {		/* read arguments */
			(*args)[i] = (double) atof (arg_list[i + 1]);
			free (arg_list[i + 1]); arg_list[i + 1] = NULL;
		}
	}
	free (arg_list); arg_list = NULL;

	return pcnt;		/* return the number of arguments */
}

/* Check if x is in the range l_limit - u_limit, and stop with error message */
/* if x is not in the specified range.					     */

void	check_value (double x, double l_limit, double u_limit, char *errmsg)
{
	if ((x < l_limit) || (x > u_limit)) err_msg_d (errmsg, x);
}

/* Convert a double value to int, and check if it is in the specified range. */

int	check_int (double x, int l_limit, int u_limit, char *errmsg)
{
	int	y;

	if ((x < (double) INT_MIN) || (x > (double) INT_MAX))
		err_msg_d (errmsg, x);		/* value is out of range */
	y = (int) (x + (x < 0.0 ? -0.5 : 0.5));
	if (fabs (x - (double) y) > 0.0001)
		err_msg_d (errmsg, x);		/* value is not an integer */
	if ((y < l_limit) || (y > u_limit))
		err_msg_i (errmsg, y);		/* value is out of range */

	return y;
}

