
/* score_write.c - score printing functions */

#include "main.h"

/* set default printing format and event mask */

void	init_p_params (void)
{
	int	i;

	/* default event mask */

	i = -1; while (++i < N_MASKS) {
		e_masks[i].m_mode = 0;
		e_masks[i].p_num = 2;
		e_masks[i].l_limit = -1.0e+36;
		e_masks[i].u_limit = 1.0e+36;
		e_masks[i].loop = -1;
	}

	/* default print format */

	for (i = 0; i < P_MAX; i++) {
		p_params[i].fwidth = 10;		/* field width	*/
		p_params[i].precision = 4;		/* precision	*/
		p_params[i].l_limit = -1.0e+36;		/* lower limit	*/
		p_params[i].u_limit = 1.0e+36;		/* upper limit	*/
	}

	/* instr number */

	p_params[0].fwidth = 1;
	p_params[0].precision = 0;
	p_params[0].l_limit = 1.0;

	/* start time */

	p_params[1].l_limit = 0.0;

	/* duration */

	p_params[2].l_limit = 0.001;

	/* note number */

	p_params[3].fwidth = 3;
	p_params[3].precision = 0;
	p_params[3].l_limit = 0.0;
	p_params[3].u_limit = 127.0;

	/* velocity */

	p_params[4].fwidth = 3;
	p_params[4].precision = 0;
	p_params[4].l_limit = 0.0;
	p_params[4].u_limit = 127.0;
}

/* print all score events */

void	print_score (void)
{
	int	i, j;
	double	x, y;

	i = -1; while ((i = next_event (i)) >= 0) {	/* i = event number  */
		if (i & 1) continue;			/* skip note-off     */
		fprintf (out_file, "\ni ");
		j = -1;				/* j = p-field number - 1 */
		while (!(read_pfield (&x, i, (++j) + 1))) {
			if (j == 1) {			/* start time	*/
				read_pfield (&y, i, 0); x += y;
			}
			if ((j == 2) && (x > 0.0)	/* duration	*/
			    && (!(read_pfield (&y, i + 1, 0)))) {
				x += y; read_pfield (&y, i, 0); x -= y;
				if (x <= 0.0) x = 1.0e-36;
			}
			if ((j != 2) || (x > 0.0)) {
				if (x < p_params[j].l_limit)
					x = p_params[j].l_limit;
				if (x > p_params[j].u_limit)
					x = p_params[j].u_limit;
			}
			if (j) fprintf (out_file, "\t");
			fprintf (out_file, "%*.*f", p_params[j].fwidth,
						    p_params[j].precision, x);
		}
	}
	fprintf (out_file, "\n\n");
}

