
/* tempomod - utility to modulate tempo of Csound scores	*/
/* written by Istvan Varga, Jul 2001				*/

#ifndef TEMPOMOD_MAIN_H
#define TEMPOMOD_MAIN_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define	MAX_PTS	(16384)			/* max. number of tempo segments     */
#define	MAX_LST	((MAX_PTS << 1) + 2)	/* max. size of tempo list	     */

extern	double	tempo[][2];	/* tempo array				*/
extern	double	t_lst[][2];	/* list of tempo values to be printed	*/
extern	double	beat_time;	/* length of tempo segments (in beats)	*/
extern	double	start_beat;	/* beat offset				*/
extern	double	avg_tempo;	/* average tempo			*/
extern	int	nr_pts;		/* number of tempo segments		*/
extern	char	**argv;		/* command line arguments		*/
extern	int	nr_args;	/* number of arguments remaining	*/
extern	int	argptr;		/* current argument			*/
extern	int	b_res;		/* time resolution in log2 of beats	*/
extern	int	nbeats;		/* number of beats			*/

#define PI_D		3.14159265358979323846

#define ISEQUAL(a, b)	(fabs ((a) - (b)) < 1.0e-5)

/* -------- functions from lib.c -------- */

/* calculate average tempo	*/

double	calc_tempo (void);

/* normalize tempo array	*/

void	normalize_tempo (void);

/* print tempo values to stdout	*/

void	print_tempo (void);

/* clear tempo data */

void	clear_tempo (void);

/* update beat_time and nr_pts */

void	update_nr_pts (void);

/* -------- functions from error.c -------- */

/* print help option to stderr */

void	opt_printf (char*);

/* print program usage */

void	print_usage (void);

/* print error message */

void	err_msg (char*, int);

/* print usage and error message */

void	err_usage (char*, int);

/* -------- functions from opts.c -------- */

/* print help */

void	opt_help (void);

/* set average tempo */

void	opt_avg_tempo (void);

/* set time offset */

void	opt_start_beat (void);

/* set number of beats */

void	opt_nbeats (void);

/* time resolution */

void	opt_b_res (void);

/* sine modulation */

void	opt_sine (void);

/* triangle / saw modulation */

void	opt_trsw (void);

#endif		/* TEMPOMOD_MAIN_H */


